@@ -15,7 +15,7 @@
 ;;; along with this program; if not, write to the Free Software
 ;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-(provide 'vm-misc)
+;;(provide 'vm-misc)
 
 (defun vm-delete-non-matching-strings (regexp list &optional destructively)
   "Delete strings matching REGEXP from LIST.
@@ -36,11 +36,13 @@ The new version of the list, minus the deleted strings, is returned."
 	  (setq curr (cdr curr)))))
     list ))
 
-(defun vm-parse (string regexp &optional matchn)
+(defun vm-parse (string regexp &optional matchn matches)
   (or matchn (setq matchn 1))
   (let (list tem)
     (store-match-data nil)
-    (while (string-match regexp string (match-end 0))
+    (while (and (not (eq matches 0))
+		(string-match regexp string (match-end 0)))
+      (and (integerp matches) (setq matches (1- matches)))
       (if (not (consp matchn))
 	  (setq list (cons (substring string (match-beginning matchn)
 				      (match-end matchn)) list))
@@ -52,7 +54,11 @@ The new version of the list, minus the deleted strings, is returned."
 					  (match-end (car tem))) list)
 		    tem nil)
 	    (setq tem (cdr tem))))))
-    (nreverse list)))
+   (if (and (integerp matches) (match-end 0)
+	    (not (eq (match-end 0) (length string))))
+       (setq list (cons (substring string (match-end 0) (length string))
+			 list)))
+   (nreverse list)))
 
 (defun vm-parse-addresses (string)
   (if (null string)
@@ -190,8 +196,7 @@ The new version of the list, minus the deleted strings, is returned."
 	(goto-char (point-max))
 	(let ((buffer-read-only nil))
 	  (insert string)))
-    (let ((temp-buffer nil)
-	  (coding-system-for-write (vm-line-ending-coding-system)))
+    (let ((temp-buffer nil))
       (unwind-protect
 	  (save-excursion
 	    (setq temp-buffer (generate-new-buffer "*vm-work*"))
@@ -201,10 +206,6 @@ The new version of the list, minus the deleted strings, is returned."
 	    ;; correct for VM's uses of this function---
 	    ;; writing out message separators
 	    (setq buffer-file-type nil)
-	    ;; Tell MULE to pick the correct newline conversion.
-	    (if (fboundp 'set-buffer-file-coding-system)
-		(set-buffer-file-coding-system 
-		 (vm-line-ending-coding-system) nil))
 	    (write-region (point-min) (point-max) where t 'quiet))
 	(and temp-buffer (kill-buffer temp-buffer))))))
 
@@ -605,29 +606,49 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
       (vm-set-extent-property ee (car props) (car (cdr props)))
       (setq props (cdr (cdr props))))))
 
-(defun vm-make-tempfile (&optional filename-suffix)
+(defun vm-make-tempfile (&optional filename-suffix proposed-filename)
   (let ((modes (default-file-modes))
-	(file (vm-make-tempfile-name filename-suffix)))
+	(file (vm-make-tempfile-name filename-suffix proposed-filename)))
     (unwind-protect
 	(progn
-	  ;; mode 600
-	  (set-default-file-modes (* 6 8 8))
+	  (set-default-file-modes (vm-octal 600))
 	  (vm-error-free-call 'delete-file file)
 	  (write-region (point) (point) file nil 0))
       (set-default-file-modes modes))
     file ))
 
-(defun vm-make-tempfile-name (&optional filename-suffix)
-  (let ((done nil) filename)
-    (while (not done)
-      (setq filename (convert-standard-filename
-		      (expand-file-name (format "vm%d%d%s"
-						vm-tempfile-counter
-						(random 100000000)
-						(or filename-suffix ""))
-					vm-temp-file-directory))
-	    vm-tempfile-counter (1+ vm-tempfile-counter)
-	    done (not (file-exists-p filename))))
+(defun vm-make-tempfile-name (&optional filename-suffix proposed-filename)
+  (let (filename)
+    (cond ((and (stringp proposed-filename)
+		(not (file-exists-p
+		      (setq filename (convert-standard-filename
+				      (expand-file-name
+				       proposed-filename
+				       vm-temp-file-directory))))))
+	   t )
+	  ((stringp proposed-filename)
+	   (let ((done nil))
+	     (while (not done)
+	       (setq filename (convert-standard-filename
+			       (expand-file-name
+				(format "%d-%s"
+					vm-tempfile-counter
+					proposed-filename)
+				vm-temp-file-directory))
+		     vm-tempfile-counter (1+ vm-tempfile-counter))
+	       done (not (file-exists-p filename)))))
+	  (t
+	   (let ((done nil))
+	     (while (not done)
+	       (setq filename (convert-standard-filename
+			       (expand-file-name
+				(format "vm%d%d%s"
+					vm-tempfile-counter
+					(random 100000000)
+					(or filename-suffix ""))
+				vm-temp-file-directory))
+		     vm-tempfile-counter (1+ vm-tempfile-counter)
+		     done (not (file-exists-p filename)))))))
     filename ))
 
 (defun vm-make-work-buffer (&optional name)
@@ -763,6 +784,40 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
       (t
        (fset 'vm-coding-system-name 'symbol-name)))
 
+(defun vm-get-file-line-ending-coding-system (file)
+  (if (not (or vm-fsfemacs-mule-p vm-xemacs-mule-p vm-xemacs-file-coding-p))
+      nil
+    (let ((coding-system-for-read  (vm-binary-coding-system))
+	  (work-buffer nil))
+      (unwind-protect
+	  (save-excursion
+	    (setq work-buffer (vm-make-work-buffer))
+	    (set-buffer work-buffer)
+	    (condition-case nil
+		(insert-file-contents file nil 0 4096)
+	      (error nil))
+	    (goto-char (point-min))
+	    (cond ((re-search-forward "[^\r]\n" nil t)
+		   (if vm-fsfemacs-mule-p 'raw-text-unix 'no-conversion-unix))
+		  ((re-search-forward "\r[^\n]" nil t)
+		   (if vm-fsfemacs-mule-p 'raw-text-mac 'no-conversion-mac))
+		  ((search-forward "\r\n" nil t)
+		   (if vm-fsfemacs-mule-p 'raw-text-dos 'no-conversion-dos))
+		  (t (vm-line-ending-coding-system))))
+	(and work-buffer (kill-buffer work-buffer))))))
+
+(defun vm-new-folder-line-ending-coding-system ()
+  (cond ((eq vm-default-new-folder-line-ending-type nil)
+	 (vm-line-ending-coding-system))
+	((eq vm-default-new-folder-line-ending-type 'lf)
+	 (if vm-fsfemacs-mule-p 'raw-text-unix 'no-conversion-unix))
+	((eq vm-default-new-folder-line-ending-type 'crlf)
+	 (if vm-fsfemacs-mule-p 'raw-text-dos 'no-conversion-dos))
+	((eq vm-default-new-folder-line-ending-type 'cr)
+	 (if vm-fsfemacs-mule-p 'raw-text-mac 'no-conversion-mac))
+	(t
+	 (vm-line-ending-coding-system))))
+
 (defun vm-collapse-whitespace ()
   (goto-char (point-min))
   (while (re-search-forward "[ \t\n]+" nil 0)
@@ -849,7 +904,7 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
   (vm-with-string-as-temp-buffer string 'vm-url-decode-buffer))
 
 (defun vm-url-decode-buffer ()
-  (let ((case-fold-search nil)
+  (let ((case-fold-search t)
 	(hex-digit-alist '((?0 .  0)  (?1 .  1)  (?2 .  2)  (?3 .  3)
 			   (?4 .  4)  (?5 .  5)  (?6 .  6)  (?7 .  7)
 			   (?8 .  8)  (?9 .  9)  (?A . 10)  (?B . 11)
@@ -879,3 +934,5 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
 	(setq size (- size (frame-pixel-width)))
 	(scroll-bar-mode nil)
 	(setq vm-fsfemacs-cached-scroll-bar-width size))))
+
+(provide 'vm-misc)
