@@ -15,7 +15,7 @@
 ;;; along with this program; if not, write to the Free Software
 ;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-(provide 'vm-reply)
+;;(provide 'vm-reply)
 
 (defun vm-do-reply (to-all include-text count)
     (let ((mlist (vm-select-marked-or-prefixed-messages count))
@@ -160,14 +160,14 @@
 	    (setq re-list (cdr re-list))))
 	result )))
 
-(defun vm-mail-yank-default (message)
+(defun vm-mail-yank-default (&optional message)
   (save-excursion
     (vm-reorder-message-headers nil vm-included-text-headers
 				vm-included-text-discard-header-regexp)
     ;; if all the headers are gone, delete the trailing blank line, too.
     (if (eq (following-char) ?\n)
 	(delete-char 1))
-    (if vm-included-text-attribution-format
+    (if (and message vm-included-text-attribution-format)
 	(let ((vm-summary-uninteresting-senders nil))
 	  (insert (vm-summary-sprintf vm-included-text-attribution-format
 				      message))))
@@ -354,9 +354,9 @@ vm-included-text-prefix is prepended to every yanked line."
 	    (mail-yank-hooks (run-hooks 'mail-yank-hooks))
 	    (t (vm-mail-yank-default message))))))
 
-(defun vm-mail-send-and-exit (arg)
-  "Just like mail-send-and-exit except that VM flags the appropriate message(s)
-as having been replied to, if appropriate."
+(defun vm-mail-send-and-exit (&rest ignored)
+  "Send message and maybe delete the composition buffer.
+The value of `vm-keep-sent-mesages' determines whether the composition buffer is deleted.  If the composition is a reply to a message ina currenttly visited folder, that message is marked as having been rpelied to."
   (interactive "P")
   (vm-check-for-killed-folder)
   (if (and (boundp 'mail-alias-file)
@@ -492,6 +492,10 @@ as having been replied to, if appropriate."
 	     nil
 	   (apply 'vm-old-select-message-coding-system ignored)))))
 
+(defvar select-safe-coding-system-function)
+
+(defvar coding-system-for-write)
+
 (defun vm-mail-send ()
   "Just like mail-send except that VM flags the appropriate message(s)
 as replied to, forwarded, etc, if appropriate."
@@ -541,10 +545,20 @@ as replied to, forwarded, etc, if appropriate."
       ;;
       ;; also protect value of this-command from minibuffer reads
       (let ((this-command this-command)
+	    ;; set up coding-system-for-write so that FCC uses
+	    ;; the correct coding system to save the message into
+	    ;; a folder.
+	    (coding-system-for-write
+	     (if (stringp mail-archive-file-name)
+		 (vm-get-file-line-ending-coding-system
+		  mail-archive-file-name)
+	       (and (boundp 'coding-system-for-write)
+		    coding-system-for-write)))
 	    ;; For Emacs 21.
 	    (mail-send-nonascii t)
 	    (sendmail-coding-system (vm-binary-coding-system))
-	    (vm-dont-ask-coding-system-question t))
+	    (vm-dont-ask-coding-system-question t)
+	    (select-safe-coding-system-function nil))
 	(save-excursion
 	  (mail-send))))
     ;; be careful, something could have killed the composition
@@ -1164,6 +1178,10 @@ found, the current buffer remains selected."
     (if (memq major-mode '(vm-mode vm-virtual-mode))
 	(setq folder-buffer (current-buffer)))
     (set-buffer (generate-new-buffer (or buffer-name "mail to ?")))
+    ;; FSF Emacs: try to prevent write-region (called to handle FCC) from
+    ;; asking the user to choose a safe coding system.
+    (if (and vm-fsfemacs-mule-p (fboundp 'set-buffer-file-coding-system))
+	(set-buffer-file-coding-system 'raw-text))
     ;; avoid trying to write auto-save files in potentially
     ;; unwritable directories.
     (setq default-directory (or vm-folder-directory (expand-file-name "~/")))
@@ -1184,7 +1202,7 @@ found, the current buffer remains selected."
 	       (nconc vm-mail-mode-map mail-mode-map)
 	       (setq vm-mail-mode-map-parented t))))
     (setq vm-mail-buffer folder-buffer
-	  mode-popup-menu (and vm-use-menus vm-popup-menu-on-mouse-3
+	  mode-popup-menu (and vm-use-menus
 			       (vm-menu-support-possible-p)
 			       (vm-menu-mode-menu)))
     (and vm-use-menus (vm-menu-support-possible-p)
@@ -1291,10 +1309,16 @@ found, the current buffer remains selected."
 		(list this-command 'composing-message))
     (if (null to)
 	(mail-position-on-field "To"))
-    (if (boundp 'post-command-idle-hook)
-	(add-hook 'post-command-idle-hook
-		   'vm-update-composition-buffer-name t)
-      (add-hook 'post-command-hook 'vm-update-composition-buffer-name t))
+    (cond ((and vm-xemacs-p
+		(fboundp 'start-itimer)
+		(null (get-itimer "vm-rename-mail"))
+	   (start-itimer "vm-rename-mail"
+			 'vm-update-composition-buffer-name
+			 1.5 1.5 t)))
+	  ((and (fboundp 'run-with-idle-timer)
+		(null vm-update-composition-buffer-name-timer))
+	   (setq vm-update-composition-buffer-name-timer
+		 (run-with-idle-timer 1.5 t 'vm-update-composition-buffer-name))))
     (run-hooks 'mail-setup-hook)))
 
 (defun vm-reply-other-frame (count)
@@ -1471,6 +1495,8 @@ message."
 	  (goto-char (point-min))
 	  (insert (vm-leading-message-separator 'mmdf))
 	  (goto-char (point-max))
+	  (if (not (eq (preceding-char) ?\n))
+	      (insert ?\n))
 	  (insert (vm-trailing-message-separator 'mmdf))
 	  (set-buffer-modified-p nil)
 	  ;; point of no return, don't kill it if the user quits
@@ -1520,3 +1546,5 @@ message."
   (remove-hook 'mail-setup-hook 'mime/editor-mode)
   (remove-hook 'mail-send-hook  'mime-edit-maybe-translate)
   (remove-hook 'mail-send-hook 'mime-editor/maybe-translate))
+
+(provide 'vm-reply)
