@@ -15,7 +15,7 @@
 ;;; along with this program; if not, write to the Free Software
 ;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-(provide 'vm-save)
+;;(provide 'vm-save)
 
 ;; (match-data) returns the match data as MARKERS, often corrupting
 ;; it in the process due to buffer narrowing, and the fact that buffers are
@@ -216,6 +216,12 @@ The saved messages are flagged as `filed'."
     (if (and (not vm-visit-when-saving) (vm-get-file-buffer folder))
 	(error "Folder %s is being visited, cannot save." folder))
     (let ((mlist (vm-select-marked-or-prefixed-messages count))
+	  (coding-system-for-write
+	   (if (file-exists-p folder)
+	       (vm-get-file-line-ending-coding-system folder)
+	     (vm-new-folder-line-ending-coding-system)))
+	  (oldmodebits (and (fboundp 'default-file-modes)
+			    (default-file-modes)))
 	  (m nil) (count 0) folder-buffer target-type)
       (cond ((and mlist (eq vm-visit-when-saving t))
 	     (setq folder-buffer (or (vm-get-file-buffer folder)
@@ -234,106 +240,111 @@ The saved messages are flagged as `filed'."
 				       (vm-message-type-of (car mlist)))))
 	    (if (eq target-type 'unknown)
 		(error "Folder %s's type is unrecognized" folder))))
-      ;; if target folder is empty or nonexistent we need to
-      ;; write out the folder header first.
-      (if mlist
-	  (let ((attrs (file-attributes folder)))
-	    (if (or (null attrs) (= 0 (nth 7 attrs)))
-		(if (null folder-buffer)
-		    (vm-write-string folder (vm-folder-header target-type))
-		  (vm-write-string folder-buffer
-				   (vm-folder-header target-type))))))
-      (save-excursion
-	(while mlist
-	  (setq m (vm-real-message-of (car mlist)))
-	  (set-buffer (vm-buffer-of m))
-	  (vm-save-restriction
-	   (widen)
-	   ;; have to stuff the attributes in all cases because
-	   ;; the deleted attribute may have been stuffed
-	   ;; previously and we don't want to save that attribute.
-	   ;; also we don't want to save out the cached summary entry.
-	   (vm-stuff-attributes m t)
-	   (if (null folder-buffer)
-	       (if (or (null vm-check-folder-types)
-		       (eq target-type (vm-message-type-of m)))
-		   (write-region (vm-start-of m)
-				 (vm-end-of m)
-				 folder t 'quiet)
-		 (if (null vm-convert-folder-types)
-		     (if (not (vm-virtual-message-p (car mlist)))
-			 (error "Folder type mismatch: %s, %s"
-				(vm-message-type-of m) target-type)
-		       (error "Message %s type mismatches folder %s"
-			      (vm-number-of (car mlist))
-			      folder
-			      (vm-message-type-of m)
-			      target-type))
-		   (vm-write-string
-		    folder
-		    (vm-leading-message-separator target-type m t))
-		   (if (eq target-type 'From_-with-Content-Length)
+      (unwind-protect
+	  (save-excursion
+	    (and oldmodebits (set-default-file-modes
+			      vm-default-folder-permission-bits))
+	    ;; if target folder is empty or nonexistent we need to
+	    ;; write out the folder header first.
+	    (if mlist
+		(let ((attrs (file-attributes folder)))
+		  (if (or (null attrs) (= 0 (nth 7 attrs)))
+		      (if (null folder-buffer)
+			  (vm-write-string folder
+					   (vm-folder-header target-type))
+			(vm-write-string folder-buffer
+					 (vm-folder-header target-type))))))
+	    (while mlist
+	      (setq m (vm-real-message-of (car mlist)))
+	      (set-buffer (vm-buffer-of m))
+	      (vm-save-restriction
+	       (widen)
+	       ;; have to stuff the attributes in all cases because
+	       ;; the deleted attribute may have been stuffed
+	       ;; previously and we don't want to save that attribute.
+	       ;; also we don't want to save out the cached summary entry.
+	       (vm-stuff-attributes m t)
+	       (if (null folder-buffer)
+		   (if (or (null vm-check-folder-types)
+			   (eq target-type (vm-message-type-of m)))
+		       (write-region (vm-start-of m)
+				     (vm-end-of m)
+				     folder t 'quiet)
+		     (if (null vm-convert-folder-types)
+			 (if (not (vm-virtual-message-p (car mlist)))
+			     (error "Folder type mismatch: %s, %s"
+				    (vm-message-type-of m) target-type)
+			   (error "Message %s type mismatches folder %s"
+				  (vm-number-of (car mlist))
+				  folder
+				  (vm-message-type-of m)
+				  target-type))
 		       (vm-write-string
 			folder
-			(concat vm-content-length-header " "
-				(vm-su-byte-count m) "\n")))
-		   (write-region (vm-headers-of m)
-				 (vm-text-end-of m)
-				 folder t 'quiet)
-		   (vm-write-string
-		    folder
-		    (vm-trailing-message-separator target-type))))
-	     (save-excursion
-	       (set-buffer folder-buffer)
-	       ;; if the buffer is a live VM folder
-	       ;; honor vm-folder-read-only.
-	       (if vm-folder-read-only
-		   (signal 'folder-read-only (list (current-buffer))))
-	       (let ((buffer-read-only nil))
-		 (vm-save-restriction
-		  (widen)
-		  (save-excursion
-		    (goto-char (point-max))
-		    (if (or (null vm-check-folder-types)
-			    (eq target-type (vm-message-type-of m)))
-			(insert-buffer-substring
-			 (vm-buffer-of m)
-			 (vm-start-of m) (vm-end-of m))
-		      (if (null vm-convert-folder-types)
-			  (if (not (vm-virtual-message-p (car mlist)))
-			      (error "Folder type mismatch: %s, %s"
-				     (vm-message-type-of m) target-type)
-			    (error "Message %s type mismatches folder %s"
-				   (vm-number-of (car mlist))
-				   folder
-				   (vm-message-type-of m)
-				   target-type))
-			(vm-write-string
-			 (current-buffer)
-			 (vm-leading-message-separator target-type m t))
-			(if (eq target-type 'From_-with-Content-Length)
+			(vm-leading-message-separator target-type m t))
+		       (if (eq target-type 'From_-with-Content-Length)
+			   (vm-write-string
+			    folder
+			    (concat vm-content-length-header " "
+				    (vm-su-byte-count m) "\n")))
+		       (write-region (vm-headers-of m)
+				     (vm-text-end-of m)
+				     folder t 'quiet)
+		       (vm-write-string
+			folder
+			(vm-trailing-message-separator target-type))))
+		 (save-excursion
+		   (set-buffer folder-buffer)
+		   ;; if the buffer is a live VM folder
+		   ;; honor vm-folder-read-only.
+		   (if vm-folder-read-only
+		       (signal 'folder-read-only (list (current-buffer))))
+		   (let ((buffer-read-only nil))
+		     (vm-save-restriction
+		      (widen)
+		      (save-excursion
+			(goto-char (point-max))
+			(if (or (null vm-check-folder-types)
+				(eq target-type (vm-message-type-of m)))
+			    (insert-buffer-substring
+			     (vm-buffer-of m)
+			     (vm-start-of m) (vm-end-of m))
+			  (if (null vm-convert-folder-types)
+			      (if (not (vm-virtual-message-p (car mlist)))
+				  (error "Folder type mismatch: %s, %s"
+					 (vm-message-type-of m) target-type)
+				(error "Message %s type mismatches folder %s"
+				       (vm-number-of (car mlist))
+				       folder
+				       (vm-message-type-of m)
+				       target-type))
 			    (vm-write-string
 			     (current-buffer)
-			     (concat vm-content-length-header " "
-				     (vm-su-byte-count m) "\n")))
-			(insert-buffer-substring (vm-buffer-of m)
-						 (vm-headers-of m)
-						 (vm-text-end-of m))
-			(vm-write-string
-			 (current-buffer)
-			 (vm-trailing-message-separator target-type)))))
-		  ;; vars should exist and be local
-		  ;; but they may have strange values,
-		  ;; so check the major-mode.
-		  (cond ((eq major-mode 'vm-mode)
-			 (vm-increment vm-messages-not-on-disk)
-			 (vm-clear-modification-flag-undos)))))))
-	   (if (null (vm-filed-flag m))
-	       (vm-set-filed-flag m t))
-	   (vm-increment count)
-	   (vm-modify-folder-totals folder 'saved 1 m)
-	   (vm-update-summary-and-mode-line)
-	   (setq mlist (cdr mlist)))))
+			     (vm-leading-message-separator target-type m t))
+			    (if (eq target-type 'From_-with-Content-Length)
+				(vm-write-string
+				 (current-buffer)
+				 (concat vm-content-length-header " "
+					 (vm-su-byte-count m) "\n")))
+			    (insert-buffer-substring (vm-buffer-of m)
+						     (vm-headers-of m)
+						     (vm-text-end-of m))
+			    (vm-write-string
+			     (current-buffer)
+			     (vm-trailing-message-separator target-type)))))
+		      ;; vars should exist and be local
+		      ;; but they may have strange values,
+		      ;; so check the major-mode.
+		      (cond ((eq major-mode 'vm-mode)
+			     (vm-increment vm-messages-not-on-disk)
+			     (vm-clear-modification-flag-undos)))))))
+	       (if (null (vm-filed-flag m))
+		   (vm-set-filed-flag m t))
+	       (vm-increment count)
+	       (vm-modify-folder-totals folder 'saved 1 m)
+	       (vm-update-summary-and-mode-line)
+	       (setq mlist (cdr mlist)))))
+	(and oldmodebits (set-default-file-modes oldmodebits)))
       (if m
 	  (if folder-buffer
 	      (progn
@@ -361,7 +372,7 @@ The saved messages are flagged as `filed'."
     (if (or (null vm-last-save-folder)
 	    (not (equal unexpanded-folder auto-folder)))
 	(setq vm-last-save-folder unexpanded-folder))
-    (if vm-delete-after-saving
+    (if (and vm-delete-after-saving (not vm-folder-read-only))
 	(vm-delete-message count))
     folder ))
 
@@ -405,6 +416,10 @@ vm-save-message instead (normally bound to `s')."
   (if (and (not vm-visit-when-saving) (vm-get-file-buffer file))
       (error "File %s is being visited, cannot save." file))
   (let ((mlist (vm-select-marked-or-prefixed-messages count))
+	(oldmodebits (and (fboundp 'default-file-modes)
+			  (default-file-modes)))
+	(coding-system-for-write
+	 (vm-get-file-line-ending-coding-system file))
 	(m nil) file-buffer)
     (cond ((and mlist (eq vm-visit-when-saving t))
 	   (setq file-buffer (or (vm-get-file-buffer file)
@@ -414,33 +429,37 @@ vm-save-message instead (normally bound to `s')."
     (if (and (not (memq (vm-get-folder-type file) '(nil unknown)))
 	     (not (y-or-n-p "This file looks like a mail folder, append to it anyway? ")))
 	(error "Aborted"))
-    (save-excursion
-      (while mlist
-	(setq m (vm-real-message-of (car mlist)))
-	(set-buffer (vm-buffer-of m))
-	(vm-save-restriction
-	 (widen)
-	 (if (null file-buffer)
-	     (write-region (vm-text-of m)
-			   (vm-text-end-of m)
-			   file t 'quiet)
-	   (let ((start (vm-text-of m))
-		 (end (vm-text-end-of m)))
-	     (save-excursion
-	       (set-buffer file-buffer)
-	       (save-excursion
-		 (let (buffer-read-only)
-		   (vm-save-restriction
-		    (widen)
-		    (save-excursion
-		      (goto-char (point-max))
-		      (insert-buffer-substring
-		       (vm-buffer-of m)
-		       start end))))))))
-	(if (null (vm-written-flag m))
-	    (vm-set-written-flag m t))
-	(vm-update-summary-and-mode-line)
-	(setq mlist (cdr mlist)))))
+    (unwind-protect
+	(save-excursion
+	  (and oldmodebits (set-default-file-modes
+			    vm-default-folder-permission-bits))
+	  (while mlist
+	    (setq m (vm-real-message-of (car mlist)))
+	    (set-buffer (vm-buffer-of m))
+	    (vm-save-restriction
+	     (widen)
+	     (if (null file-buffer)
+		 (write-region (vm-text-of m)
+			       (vm-text-end-of m)
+			       file t 'quiet)
+	       (let ((start (vm-text-of m))
+		     (end (vm-text-end-of m)))
+		 (save-excursion
+		   (set-buffer file-buffer)
+		   (save-excursion
+		     (let (buffer-read-only)
+		       (vm-save-restriction
+			(widen)
+			(save-excursion
+			  (goto-char (point-max))
+			  (insert-buffer-substring
+			   (vm-buffer-of m)
+			   start end))))))))
+	     (if (null (vm-written-flag m))
+		 (vm-set-written-flag m t))
+	     (vm-update-summary-and-mode-line)
+	     (setq mlist (cdr mlist)))))
+      (and oldmodebits (set-default-file-modes oldmodebits)))
     (if m
 	(if file-buffer
 	    (message "Message%s written to buffer %s" (if (/= 1 count) "s" "")
@@ -604,3 +623,58 @@ Output, if any, is displayed.  The message is not altered."
     (vm-display nil nil '(vm-print-message) '(vm-print-message))
     (if (not (zerop (save-excursion (set-buffer buffer) (buffer-size))))
 	(display-buffer buffer))))
+
+(defun vm-save-message-to-imap-folder (folder &optional count)
+  "Save the current message to an IMAP folder.
+Prefix arg COUNT means save this message and the next COUNT-1
+messages.  A negative COUNT means save this message and the
+previous COUNT-1 messages.
+
+When invoked on marked messages (via vm-next-command-uses-marks),
+all marked messages in the current folder are saved; other messages are
+ignored.
+
+The saved messages are flagged as `filed'."
+  (interactive
+   (save-excursion
+     (vm-session-initialization)
+     (vm-check-for-killed-folder)
+     (vm-select-folder-buffer-if-possible)
+     (let ((this-command this-command)
+	   (last-command last-command))
+       (list (vm-read-imap-folder-name "Save to IMAP folder: "
+				       vm-imap-server-list t)
+	     (prefix-numeric-value current-prefix-arg)))))
+  (vm-select-folder-buffer)
+  (vm-check-for-killed-summary)
+  (vm-error-if-folder-empty)
+  (vm-display nil nil '(vm-save-message-to-imap-folder)
+	      '(vm-save-message-to-imap-folder))
+  (or count (setq count 1))
+  (let ((mlist (vm-select-marked-or-prefixed-messages count))
+	process m 
+	(mailbox (nth 3 (vm-imap-parse-spec-to-list folder)))
+	(count 0))
+    (unwind-protect
+	(save-excursion
+	  (setq process (vm-imap-make-session folder))
+	  (set-buffer (process-buffer process))
+	  (while mlist
+	    (setq m (car mlist))
+	    (vm-imap-save-message process m mailbox)
+	    (if (null (vm-filed-flag m))
+		(vm-set-filed-flag m t))
+	    (vm-increment count)
+	    (vm-modify-folder-totals folder 'saved 1 m)
+	    (setq mlist (cdr mlist))))
+      (and process (vm-imap-end-session process)))
+    (vm-update-summary-and-mode-line)
+    (if (interactive-p)
+	(message "%d message%s saved to %s"
+		 count (if (/= 1 count) "s" "")
+		 (vm-safe-imapdrop-string folder)))
+    (if (and vm-delete-after-saving (not vm-folder-read-only))
+	(vm-delete-message count))
+    folder ))
+
+(provide 'vm-save)
